@@ -93,14 +93,14 @@ class MalformedQuery(Exception):
 def add_query_argument(cmd, parser):
     return parser.add_argument(
         cmd,
-        help="A query to filter resource. "
-        "The syntax is a combination of attribute, operator and value. "
-        "For example: id=90d58eea-70d7-4294-a49a-170dcdf44c3c would filter "
-        "resource with a certain id. More complex queries can be built, "
-        "e.g.: not (flavor_id!=\"1\" and memory>=24). "
-        "Use \"\" to force data to be interpreted as string. "
-        "Supported operators are: not, and, ∧ or, ∨, >=, <=, !=, >, <, =, ==, "
-        "eq, ne, lt, gt, ge, le, in, like, ≠, ≥, ≤, like, in.",
+        help=u"A query to filter resource. "
+        u"The syntax is a combination of attribute, operator and value. "
+        u"For example: id=90d58eea-70d7-4294-a49a-170dcdf44c3c would filter "
+        u"resource with a certain id. More complex queries can be built, "
+        u"e.g.: not (flavor_id!=\"1\" and memory>=24). "
+        u"Use \"\" to force data to be interpreted as string. "
+        u"Supported operators are: not, and, ∧ or, ∨, >=, <=, !=, >, <, =, "
+        u"==, eq, ne, lt, gt, ge, le, in, like, ≠, ≥, ≤, like, in.",
         type=search_query_builder)
 
 
@@ -177,7 +177,7 @@ def dict_from_parsed_args(parsed_args, attrs):
 
 def dict_to_querystring(objs):
     strings = []
-    for k, values in objs.items():
+    for k, values in sorted(objs.items()):
         if values is not None:
             if not isinstance(values, (list, tuple)):
                 values = [values]
@@ -188,6 +188,35 @@ def dict_to_querystring(objs):
     return "&".join(strings)
 
 
+def get_pagination_options(parsed_args):
+    options = dict(
+        sorts=parsed_args.sort,
+        limit=parsed_args.limit,
+        marker=parsed_args.marker)
+
+    if hasattr(parsed_args, 'details'):
+        options['details'] = parsed_args.details
+    if hasattr(parsed_args, 'history'):
+        options['history'] = parsed_args.history
+    return options
+
+
+def build_pagination_options(details=False, history=False,
+                             limit=None, marker=None, sorts=None):
+    options = []
+    if details:
+        options.append("details=true")
+    if history:
+        options.append("history=true")
+    if limit:
+        options.append("limit=%d" % limit)
+    if marker:
+        options.append("marker=%s" % urllib_parse.quote(marker))
+    for sort in sorts or []:
+        options.append("sort=%s" % urllib_parse.quote(sort))
+    return "%s" % "&".join(options)
+
+
 # uuid5 namespace for id transformation.
 # NOTE(chdent): This UUID must stay the same, forever, across all
 # of gnocchi to preserve its value as a URN namespace.
@@ -210,6 +239,10 @@ def encode_resource_id(value):
 
 
 def get_client(obj):
-    # TODO(sileht): return the location of Gnocchi client when the app is the
-    # OSC one instead of the Gnocchi one
-    return obj.app.client
+    if hasattr(obj.app, 'client_manager'):
+        # NOTE(sileht): cliff objects loaded by OSC
+        return obj.app.client_manager.metric
+    else:
+        # TODO(sileht): Remove this when OSC is able
+        # to install the gnocchi client binary itself
+        return obj.app.client
